import { Signal } from "@preact/signals";
declare type Signalable = Array<any> | ((...args: any[]) => any) | string | boolean | number | bigint | symbol | undefined | null;
declare type Storeable = {
    [key: string]: (() => any) | Signalable | Storeable;
};
declare type ReadOnlyDeep<T> = {
    readonly [P in keyof T]: ReadOnlyDeep<T[P]>;
};
export interface IDeepSignal<T extends Storeable> {
    value: ReadOnlyDeep<T>;
    peek: () => ReadOnlyDeep<T>;
}
export declare type DeepSignal<T extends Storeable> = IDeepSignal<T> & {
    [K in keyof T]: T[K] extends Signalable ? Signal<T[K]> : T[K] extends Storeable ? DeepSignal<T[K]> : Signal<T[K]>;
};
export declare const deepSignal: <T extends Storeable>(initialValue: T) => DeepSignal<T>;
export declare const useDeepSignal: <T extends Storeable>(initial: T | (() => T)) => DeepSignal<T>;
export {};
